
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

#include "SMARTctl.h"

int print_SMART_info(char *HDD,SMARTINFO_t *pSMARTinfo)
{
	time_t tim;
	unsigned short i;
	int i_pass = 0;


	time(&tim);

	printf("S.M.A.R.T READ-DEVICE : %s\n",HDD);
	printf("S.M.A.R.T READ-DATE   : %s",ctime(&tim));
	printf("S.M.A.R.T REVISION    : 0x%04X\n",pSMARTinfo->revision);
	printf("S.M.A.R.T CAPABILITY  : 0x%04X\n",pSMARTinfo->capability);
	printf("S.M.A.R.T DATA-COUNT  : %u\n",pSMARTinfo->datacount);
	printf("-----------------------------------\n");
	printf(" ID VALUE THRESHOLD STATUS CHECK\n");
	printf("-----------------------------------\n");
	for(i = 0;i < pSMARTinfo->datacount;i++)
	{
#if 0
		pSMARTinfo->data[i].value = pSMARTinfo->data[i].threshold;	/* DEBUG */
#endif
		printf("%3u  %3u    %3u     0x%04X ",
			(unsigned int)(pSMARTinfo->data[i].id),
			(unsigned int)(pSMARTinfo->data[i].value),
			(unsigned int)(pSMARTinfo->data[i].threshold),
			(unsigned int)(pSMARTinfo->data[i].status) );
		if(pSMARTinfo->data[i].value > pSMARTinfo->data[i].threshold)
		{
			printf("Passed\n");
		}
		else
		{
			printf("**Failed**\n");
			i_pass = 1;
		}
	}

	printf("-----------------------------------\n");

	if(i_pass != 0)
	{
		fprintf(stderr,"** Found S.M.A.R.T failed : %s **\n",HDD);
	}

	return i_pass;
}

int main(int argc,char *argv[])
{
	int c;
	int i_retval = 0;
	char *HDD = NULL;
	int i_query = 0;
	SMARTINFO_t SMARTinfo;
	int fd;
	int status;


	do {

		c = getopt(argc,argv,"d:q");

		if(c != EOF)
		{
			switch(c)
			{
				case 'd':	/* Target HDD */
					HDD = optarg;
					break;
				case 'q':	/* query S.M.A.R.T */
					i_query = 1;
					break;
				default:
					c = EOF;
					break;
			}
		}

	} while (c != EOF);

	if((i_query != 0) && (HDD != NULL))
	{
		if((fd = open(HDD, O_RDONLY)) >= 0)
		{
			if((status = SMART_read(fd,&SMARTinfo)) == SMARTERR_NOERR)
			{
				if(print_SMART_info(HDD,&SMARTinfo) != 0)
				{
					i_retval = 30;	/* S.M.A.R.T warning or error. */
				}
			}
			else
			{
				fprintf(stderr,"S.M.A.R.T read error [%d].\n",status);
				i_retval = 20;	/* Can't Read Device. */
			}

			close(fd);
		}
		else
		{
			perror(HDD);
			i_retval = 10;	/* Can't Open Device. */
		}
	}

	printf("ERROR EXIT : %d\n",i_retval);

	return i_retval;
}

