
/* $Id$ */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <linux/hdreg.h>
#include <linux/types.h>

#include "SMARTctl.h"

static int SMART_testsum(unsigned char *pdata)
{
	unsigned char by_calcsum = 0;
	int i;

	for(i = 0;i < SMART_DATASECT_SIZE;i++)
	{
		by_calcsum += *(pdata + i);
	}

	return (int)by_calcsum;	/* 0:OK else CheckSUM Err. */
}

static unsigned short SMART_get_ushort(unsigned char *pdata)
{
	unsigned short ui_retval = 0;

	ui_retval = (*(pdata + 1));
	ui_retval = (ui_retval << 8) | (*(pdata + 0));

	return ui_retval;
}

static int SMART_buildinfo(unsigned char *pvalues,unsigned char *pthresholds,SMARTINFO_t *pSMARTinfo)
{
	int i;
	int v_ofs,t_ofs;

	/* get revision */
	pSMARTinfo->revision = SMART_get_ushort(pvalues);

	v_ofs = 2;
	t_ofs = 2;
	for(i = 0;i < SMARTCTL_INFO_MAX;i++)
	{
		/* Check ID (value & thresholds) */
		if( (*(pvalues + v_ofs) != 0x00) 			/* Valid ID */
		    && (*(pthresholds + t_ofs) != 0x00) )		/* Valid ID */
		{
			if(*(pvalues + v_ofs) == *(pthresholds + t_ofs))
			{
				pSMARTinfo->data[i].id         = *(pvalues + v_ofs);
				pSMARTinfo->data[i].value      = *(pvalues + v_ofs + 3);
				pSMARTinfo->data[i].threshold  = *(pthresholds + t_ofs + 1);
				pSMARTinfo->data[i].worst      = *(pvalues + v_ofs + 4);
				pSMARTinfo->data[i].status     = SMART_get_ushort(pvalues + v_ofs + 1);
				pSMARTinfo->data[i].reserved   = 0;
			}
			else
			{
				/* This Data is reject. */
			}
		}
		else
		{
			break;	/* Build end.*/
		}

		v_ofs += 12;
		t_ofs += 12;
	}

	pSMARTinfo->datacount = i;

	pSMARTinfo->capability = SMART_get_ushort(pvalues + 2 + (12 * SMARTCTL_INFO_MAX) + 6);

	return SMARTERR_NOERR;
}

int SMART_read(int fd,SMARTINFO_t *pSMARTinfo)
{
	int i_retval = SMARTERR_INTERNAL;
	unsigned char SMART_COMMAND_enable[4]                         = { WIN_SMART, 0, SMART_ENABLE         , 0 };
	unsigned char SMART_COMMAND_disable[4]                        = { WIN_SMART, 0, SMART_DISABLE        , 0 };
	unsigned char SMART_COMMAND_values[4+SMART_DATASECT_SIZE]     = { WIN_SMART, 0, SMART_READ_VALUES    , 1 };
	unsigned char SMART_COMMAND_thresholds[4+SMART_DATASECT_SIZE] = { WIN_SMART, 0, SMART_READ_THRESHOLDS, 1 };

	i_retval = SMARTERR_IOERR;

	if(ioctl(fd, HDIO_DRIVE_CMD, &SMART_COMMAND_enable) == 0)	/* S.M.A.R.T enable : OK */
	{
		if( (ioctl(fd, HDIO_DRIVE_CMD, &SMART_COMMAND_values) == 0)		/* S.M.A.R.T read-values : OK */
		    && (ioctl(fd, HDIO_DRIVE_CMD, &SMART_COMMAND_thresholds) == 0) )	/* S.M.A.R.T read-thresholds : OK */
		{

			i_retval = SMARTERR_BADSUM;

			if( (SMART_testsum(&(SMART_COMMAND_values[4])) == 0)		/* S.M.A.R.T read-values CheckSUM : OK */
			    && (SMART_testsum(&(SMART_COMMAND_thresholds[4])) == 0) )	/* S.M.A.R.T read-thresholds CheckSUM : OK */
			{
				/* OK. S.M.A.R.T-Infomation is valid. */

				i_retval = SMART_buildinfo(&(SMART_COMMAND_values[4]),&(SMART_COMMAND_thresholds[4]),pSMARTinfo);

			}
		}

		(void)ioctl(fd, HDIO_DRIVE_CMD, &SMART_COMMAND_disable);	/* S.M.A.R.T disable */
	}

	return i_retval;
}

